package com.xiaqiuchu.common.config.shiro;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.servlet.Filter;

import org.apache.shiro.realm.Realm;
import org.apache.shiro.spring.web.ShiroFilterFactoryBean;
import org.apache.shiro.web.mgt.DefaultWebSecurityManager;
import org.springframework.aop.framework.autoproxy.DefaultAdvisorAutoProxyCreator;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.apache.shiro.mgt.DefaultSessionStorageEvaluator;
import org.apache.shiro.mgt.DefaultSubjectDAO;
import org.apache.shiro.mgt.SecurityManager;

@Configuration
public class ShiroConfig {

    // //创建自定义Realm
    @Bean
    public Realm realm() {
        return new ShiroJwtRealm();
    }

    // 创建安全管理器
    @Bean("securityManager")
    public DefaultWebSecurityManager getDefaultWebSecurityManager() {
        DefaultWebSecurityManager securityManager = new DefaultWebSecurityManager();
        securityManager.setRealm(realm());
        /*
         * 复制自jeecg-boot
         * 关闭shiro自带的session，详情见文档
         * http://shiro.apache.org/session-management.html#SessionManagement-
         * StatelessApplications%28Sessionless%29
         */
        // DefaultSubjectDAO subjectDAO = new DefaultSubjectDAO();
        // DefaultSessionStorageEvaluator defaultSessionStorageEvaluator = new DefaultSessionStorageEvaluator();
        // defaultSessionStorageEvaluator.setSessionStorageEnabled(false);
        // subjectDAO.setSessionStorageEvaluator(defaultSessionStorageEvaluator);
        // securityManager.setSubjectDAO(subjectDAO);
        //
        return securityManager;
    }

    @Bean("shiroFilterFactoryBean")
    public ShiroFilterFactoryBean shiroFilterFactoryBean(SecurityManager securityManager) {
        ShiroFilterFactoryBean shiroFilter = new ShiroFilterFactoryBean();
        shiroFilter.setSecurityManager(getDefaultWebSecurityManager());
        // 未授权跳转
        shiroFilter.setUnauthorizedUrl("/index/unauthorized");
        // 登录地址（未登录则自动重定向到当前）
        // shiroFilter.setLoginUrl("/index/login");
        //
        Map<String, Filter> filterMap = new HashMap<>();
        //
        filterMap.put("jwt", new ShiroJwtFilter());
        //
        shiroFilter.setFilters(filterMap);

        // 拦截器 以及为什么用 linkedhashmap https://blog.csdn.net/sgx5666666/article/details/109276397
        Map<String, String> filterRuleMap = new LinkedHashMap<>();
        // 匿名访问
        filterRuleMap.put("/index/index", "anon");
        filterRuleMap.put("/index/login", "anon");
        filterRuleMap.put("/index/unauthorized", "anon");
        // 登录并具有 admin 角色
        // filterRuleMap.put("/index/admin", "authc,roles[admin]");
        // filterRuleMap.put("/index/admin", "jwt,roles[admin]");
        // 通过jwt校验，需登录才能访问（自行实现逻辑）
        filterRuleMap.put("/**", "jwt");
        //
        shiroFilter.setFilterChainDefinitionMap(filterRuleMap);
        //
        return shiroFilter;
    }


    // creator.setUsePrefix(true) 在这里是解决另一个问题的。因为当为false时，realm里的doGetAuthorizationInfo会执行两次。另外setProxyTargetClass(true)像是多余的，springboot2.0之后默认使用cglib代理，不需要显示声明为true。
    // @Bean
    // public static DefaultAdvisorAutoProxyCreator getDefaultAdvisorAutoProxyCreator(){
    //     DefaultAdvisorAutoProxyCreator defaultAdvisorAutoProxyCreator=new DefaultAdvisorAutoProxyCreator();
    //     defaultAdvisorAutoProxyCreator.setUsePrefix(true);
    //     return defaultAdvisorAutoProxyCreator;
    // }
}
