package com.xiaqiuchu.common.config.shiro;

import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import org.apache.shiro.web.filter.AccessControlFilter;
import lombok.extern.slf4j.Slf4j;

/*
 * 自定义一个Filter，用来拦截所有的请求判断是否携带Token
 * isAccessAllowed()判断是否携带了有效的JwtToken
 * onAccessDenied()是没有携带JwtToken的时候进行账号密码登录，登录成功允许访问，登录失败拒绝访问
 * */
@Slf4j
public class ShiroJwtFilter extends AccessControlFilter {

    ShiroJwtFilter(){}

    /*
     * 判断是否通过访问，如果不通过走 onAccessDenied。
     */
    @Override
    protected boolean isAccessAllowed(ServletRequest request, ServletResponse response, Object mappedValue)
            throws Exception {
        log.info("执行类："+this.getClass().getName());
        log.info("执行方法："+Thread.currentThread() .getStackTrace()[1].getMethodName());
        // 取出token并校验真实性（还应该校验一下是否过期）
        HttpServletRequest httpServletRequest   = (HttpServletRequest) request;
        String jwt                              = httpServletRequest.getHeader("Authorization");
        // 返回false来使用onAccessDenied()方法
        // 本类没有标记为@Component，所以无法自动注入。
        if(new ShiroJwtUtil().verify(jwt)){
            getSubject(request, response).login(new ShiroJwtToken(jwt));
            return true;
        }
        return false;
    }

    /**
     * 不通过处理，如果返回true，依旧可以通过。
     */
    @Override
    protected boolean onAccessDenied(ServletRequest servletRequest, ServletResponse servletResponse) throws Exception {
        log.info("执行类："+this.getClass().getName());
        log.info("执行方法："+Thread.currentThread() .getStackTrace()[1].getMethodName());
        //
        throw new RuntimeException("身份验证异常");
    }
}
