package com.xiaqiuchu.common.config.shiro;

import java.util.HashSet;
import org.apache.shiro.authc.AuthenticationException;
import org.apache.shiro.authc.AuthenticationInfo;
import org.apache.shiro.authc.AuthenticationToken;
import org.apache.shiro.authc.SimpleAuthenticationInfo;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.subject.PrincipalCollection;
import org.springframework.stereotype.Component;

import cn.hutool.jwt.JWT;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Component
public class ShiroJwtRealm extends AuthorizingRealm {

    /*
     * 多重写一个support
     * 标识这个Realm是专门用来验证JwtToken
     * 不负责验证其他的token（UsernamePasswordToken）
     * */
    @Override
    public boolean supports(AuthenticationToken token) {
        //这个token就是从过滤器中传入的jwtToken
        return token instanceof ShiroJwtToken;
    }

    /**
     * 用户认证
     */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token) throws AuthenticationException {
        log.info("执行类："+this.getClass().getName());
        log.info("执行方法："+Thread.currentThread() .getStackTrace()[1].getMethodName());
        // 在 ShiroJwtFilter 中存入的 Principal，也就是 getSubject(request, response).login(new ShiroJwtToken(jwt));
        String jwt = token.getPrincipal().toString();
        // token可以被伪造，但是已经在 isAccessAllowed 中校验过 jwt了，所以这里信任提取数据即可
        JWT jwtObj = new ShiroJwtUtil().parseToken(jwt);
        // 数据库查询、校验用户是否存在等等、、、
        jwtObj.getPayload("id");
        // 返回一个shiro用户。
        return new SimpleAuthenticationInfo(jwt,jwt, this.getClass().getName());
        //这里返回的是类似账号密码的东西，但是jwtToken都是jwt字符串。还需要一个该Realm的类名

    }
    /**
     * 权限认证（doGetAuthenticationInfo通过后执行权限获取）
     */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principals) {
        log.info("执行类："+this.getClass().getName());
        log.info("执行方法："+Thread.currentThread() .getStackTrace()[1].getMethodName());
        // 获取用户,获取用户权限，授权给 shiro用户类
        String jwt = principals.getPrimaryPrincipal().toString();
        // 授权信息可以走数据库
        SimpleAuthorizationInfo simpleAuthorizationInfo = new SimpleAuthorizationInfo();
        // 设置角色
        simpleAuthorizationInfo.setRoles(new HashSet<String>() {{
            add("admin");
            add("user");
        }});
        // 设置权限
        simpleAuthorizationInfo.addStringPermission("admin:*");
        // log.info("token"+token);
        return simpleAuthorizationInfo;
    }

    
}
